var LICENSE_SERVER_URL = "https://config.gravity.global/gravity/";
var SERVER_URL = LICENSE_SERVER_URL + "services/extension/config?url=" + encodeURI(location.href);
var CONFIG_LIST_URL = LICENSE_SERVER_URL + "services/extension/config/list";
var SITES_MAPPINGS_URL = "services/extension/sites/mappings";

var STARTS_WITH = 'STARTS_WITH';
var PAGE = 'PAGE';
var DOMAIN = 'DOMAIN';

var STRIPPED_WWW = 'www.';

var chatty = true;

(function init() {
    chrome.storage.local.get(['gravityConsoleLevel'], function (result) {
        chatty = result && result.gravityConsoleLevel;
        chatter('Verbose logging is ON. Visit the extension options page to turn if off.');
    });
    chatter('Starting gravity extension.');
    checkBaseUrl();
})();

function checkConfigList() {
    chatter('Trying to read config list from chrome local storage');
    chrome.storage.local.get(["configList", "configListLifeTime", "configListCreationTime"], function (response) {
        chatter('Fetched config list, validating...');
        if (response.configList && response.configList.length && isDataValid(response.configListLifeTime, response.configListCreationTime)) {
            chatter('Config from storage is valid, check if url is suitable:', response.configList, response.configListLifeTime, response.configListCreationTime);
            if (isUrlSuitable(response.configList)) {
                chatter('Url is suitable.');
                getServerBaseUrl();
            } else {
                chatter('Check config list: Url is *NOT* suitable.');
                useOldBaseUrlForSiteMappings();
            }
        } else {
            getConfigList();
        }
    });
}

function getConfigList() {
    chatter('Trying to get new config list from', CONFIG_LIST_URL);

    chrome.runtime.sendMessage(
        {contentScriptQuery: "getConfigList", url: CONFIG_LIST_URL},
        function (response) {
            chatter('Got new config list', response);
            saveConfigListToExtensionStorage(response.configList, response.configListLifeTime);
            chatter('Config retrieved, check if url is suitable:', response.configList, response.configListLifeTime);
            if (isUrlSuitable(response.configList)) {
                chatter('Url is suitable.');
                getServerBaseUrl();
            } else {
                useOldBaseUrlForSiteMappings();
            }
        }
    );
}

function isUrlSuitable(configList) {
    var matchIndex = findMatch(configList);
    if (matchIndex === -1) {
        return false;
    }

    var whitelistMatch = configList[matchIndex];
    if (!whitelistMatch.blackList) {
        return true;
    }

    var blacklistMatch = findMatch(whitelistMatch.blackList);
    if (blacklistMatch === -1) {
        return true;
    }
    chatter('Blacklisted ', location.href);
    return false;
}

function findMatch(configList) {
    var startsWithUrl = getStartsWithUrl();
    var pageUrl = getPageUrl();
    var urlDomain = getUrlDomain();

    var hashedUrl;
    var bitArray;
    for (var i = 0; i < configList.length; i++) {
        var hashType = configList[i].hashType;
        switch (true) {
            case hashType.indexOf(STARTS_WITH) > -1:
            {
                var splitted = hashType.split('_');
                var urlLength = Number(splitted[splitted.length - 1]);
                bitArray = sjcl.hash.sha256.hash(startsWithUrl.substring(0, urlLength));
                hashedUrl = sjcl.codec.hex.fromBits(bitArray);
                break;
            }
            case hashType.indexOf(PAGE) > -1:
            {
                bitArray = sjcl.hash.sha256.hash(pageUrl);
                hashedUrl = sjcl.codec.hex.fromBits(bitArray);
                break;
            }
            case hashType.indexOf(DOMAIN) > -1:
            {
                bitArray = sjcl.hash.sha256.hash(urlDomain);
                hashedUrl = sjcl.codec.hex.fromBits(bitArray);
                break;
            }
        }

        if (hashedUrl === configList[i].hash) {
            chatter('Found match, since url\'s are hashed, no match info can be given.');
            return i;
        }
    }

    return -1;
}

function useOldBaseUrlForSiteMappings() {
    chrome.storage.local.get(["baseUrl"], function (response) {
        if (response.baseUrl) {
            chatter('#################### Cached base url was used. Mapping list will be checked');
            checkMappingList(response.baseUrl);
        }
    });
}

function checkBaseUrl() {
    chrome.storage.local.get(["baseUrl", "baseUrlLifeTime", "baseUrlCreationTime"], function (response) {
        if (response.baseUrl && isDataValid(response.baseUrlLifeTime, response.baseUrlCreationTime)) {
            checkMappingList(response.baseUrl);
        } else {
            checkConfigList()
        }
    });
}

function getServerBaseUrl() {
    chatter('Trying to get new base url from', SERVER_URL);

    chrome.runtime.sendMessage(
        {contentScriptQuery: "getServerBaseUrl", url: SERVER_URL},
        function (response) {
            chatter('Got new base url', response);
            var baseUrl = response.serverUrl;
            var baseUrlLifeTime = response.serverUrlLifeTime;
            if (baseUrl && baseUrlLifeTime) {
                chatter('Base URL is valid:', baseUrl, baseUrlLifeTime);
                saveBaseUrlInfoToExtensionStorage(baseUrl, baseUrlLifeTime);
                checkMappingList(baseUrl)
            } else {
                useOldBaseUrlForSiteMappings();
            }
        }
    );
}

function checkMappingList(baseUrl) {
    chatter('Trying to read mapping list from chrome local storage');
    chrome.storage.local.get(["mappingList", "sitesMappingsLifeTime", "sitesMappingsCreationTime"], function (response) {
        chatter('Fetched mapping list, validating...');
        if (response.mappingList && response.mappingList.length && isDataValid(response.sitesMappingsLifeTime, response.sitesMappingsCreationTime)) {
            chatter('Sites mappings from storage is valid, check if url is suitable:', response.mappingList, response.sitesMappingsLifeTime, response.sitesMappingsCreationTime);
            if (isUrlSuitable(response.mappingList)) {
                chatter('Url is suitable.');
                loadInjectScripts(baseUrl);
            } else {
                chatter('Url is *NOT* suitable.', location.href);
            }
        } else {
            getSitesMappings(baseUrl);
        }
    });
}

function getSitesMappings(baseUrl) {
    chatter('Trying to get site mapping from', baseUrl + SITES_MAPPINGS_URL);

    chrome.runtime.sendMessage(
        {contentScriptQuery: "getSiteMappings", url: baseUrl + SITES_MAPPINGS_URL},
        function (response) {
            chatter('Got new sites mappings', response);
            saveMappingListToExtensionStorage(response.mappingList, response.sitesMappingsLifeTime);
            if (isUrlSuitable(response.mappingList)) {
                chatter('Url is suitable.');
                loadInjectScripts(baseUrl);
            }
        }
    );
}

function loadInjectScripts(baseUrl) {
    chatter('Loading inject scripts from', baseUrl);
    var scriptUrl = baseUrl + "inject/";
    var browserExtensionInfo = "Chrome extension - " + chrome.runtime.getManifest().version;
    var script = document.createElement('script');
    script.onload = function () {
        var startScript = document.createElement('script');
        var srcBody = document.createTextNode("pi.gravity.inject.setup('" + baseUrl + "', '" + scriptUrl + "', '" + browserExtensionInfo +"')");
        startScript.appendChild(srcBody);
        document.head.appendChild(startScript);
    };
    script.src = scriptUrl + "js/pi.gravity.inject.js";
    document.head.appendChild(script);
}

function saveBaseUrlInfoToExtensionStorage(baseUrl, baseUrlLifeTime) {
    var container = {
        "baseUrl": baseUrl,
        "baseUrlLifeTime": baseUrlLifeTime,
        "baseUrlCreationTime": new Date().getTime()
    };
    chrome.storage.local.set(container, function () {
        chatter('Base URL info saved to chrome.storage.local', container);
    });

}

function saveConfigListToExtensionStorage(configList, configListLifeTime) {
    var container = {
        "configList": configList,
        "configListLifeTime": configListLifeTime,
        "configListCreationTime": new Date().getTime()
    };
    chrome.storage.local.set(container, function () {
        chatter('Site config saved to chrome.storage.local', container);
    });
}

function saveMappingListToExtensionStorage(mappingList, sitesMappingsLifeTime) {
    var container = {
        "mappingList": mappingList,
        "sitesMappingsLifeTime": sitesMappingsLifeTime,
        "sitesMappingsCreationTime": new Date().getTime()
    };
    chrome.storage.local.set(container, function () {
        chatter('Sites mappings saved to chrome.storage.local', container);
    });
}

function getUrlDomain() {
    var url = location.hostname;
    return stripWWW(url);
}

function getPageUrl() {
    var url = location.hostname + location.pathname;
    return stripWWW(url);
}

function getStartsWithUrl() {
    var url = location.href.substring((location.protocol + '//').length);
    return stripWWW(url);
}

function stripWWW (url) {
    if (url.substring(0, 4) === STRIPPED_WWW) {
        url = url.substring(STRIPPED_WWW.length);
    }
    return url
}

function isDataValid(baseUrlLifeTime, baseUrlCreationTime) {
    var currentTimestamp = new Date().getTime();
    var dataValid = currentTimestamp - baseUrlCreationTime < baseUrlLifeTime;
    chatter(dataValid ? 'Data is valid/not expired' : 'Data is *EXPIRED*');
    return dataValid;
}

function chatter() {
    if (chatty) {
        console.log.apply(this, arguments);
    }
}
